@@ -200,8 +200,8 @@ Warning: dim_name_map may be changed in place
                 bounds._change_axis_names(dim_name_map)
     #--- End: def
 
-    def _equivalent_data(self, other, rtol=None, atol=None, traceback=False,
-                         copy=True):
+    def _equivalent_data(self, other, rtol=None, atol=None,
+                         traceback=False, copy=True):
         '''
 
 :Parameters:
@@ -230,16 +230,30 @@ Warning: dim_name_map may be changed in place
 
         if (self.direction() != other.direction() and 
             self.direction() is not None and other.direction() is not None):
-            if copy:
-                other = other.copy()
-                copy = False
-            other.flip(i=True)
+            other = other.flip(i=not copy)
+            copy = False
         #--- End: if            
 
+# DCH - should we test self.cyclic(), too? Think of non-cycli yet period coords ....
+
+#        period = self._period
+#        if period != other._period:
+#            return False
+
         # Compare the data arrays
         if not super(Coordinate, self)._equivalent_data(other, rtol=rtol,
                                                         atol=atol, copy=copy):
-            return False
+            return False 
+#            if period is None:
+#                return False
+#
+#            other = other.anchor(self.datum(0), i=not copy)
+#            copy = False
+#
+#            if not super(Coordinate, self)._equivalent_data(other, rtol=rtol,
+#                                                            atol=atol, copy=copy):
+#                return False
+        #--- End: if
 
         if self._hasbounds:
             # Both coordinates have bounds
@@ -2565,6 +2579,75 @@ array([5, 3, 1])
         return self.bounds.data[..., i].squeeze(1, i=True)
     #--- End: def
 
+#    def anchor(self, value, i=False, dry_run=False):  
+#        '''
+#        '''
+#        if i or dry_run:
+#            c = self
+#        else:
+#            c = self.copy()
+#        
+#        
+#        period = c.period()
+#        if period is None:
+#            raise ValueError(
+#"Cyclic {!r} axis has no period".format(dim.name()))
+#
+#        value = Data.asdata(value)
+#        if not value.Units:
+#            value = value.override_units(dim.Units)
+#        elif not value.Units.equivalent(dim.Units):
+#            raise ValueError(
+#"Anchor value has incompatible units: {!r}".format(value.Units))
+#
+#        axis_size = self.size
+#        if axis_size <= 1:
+#            # Don't need to roll a size one axis
+#            if dry_run:
+#                return {'axis': axis, 'roll': 0, 'nperiod': 0}
+#            else:
+#                return c
+#        
+#        d = c.data
+#
+#        if c.increasing:
+#            # Adjust value so it's in the range [d[0], d[0]+period) 
+#            n = ((d[0] - value) / period).ceil(i=True)
+#            value1 = value + n * period
+#
+#            shift = axis_size - numpy_argmax(d - value1 >= 0)
+#            if not dry_run:
+#                c.roll(axis, shift, i=True)     
+#
+#            dim = domain.item(axis)
+#            n = ((value - c.data[0]) / period).ceil(i=True)
+#        else:
+#            # Adjust value so it's in the range (d[0]-period, d[0]]
+#            n = ((d[0] - value) / period).floor(i=True)
+#            value1 = value + n * period
+#
+#            shift = axis_size - numpy_argmax(value1 - d >= 0)
+#            if not dry_run:
+#                c.roll(axis, shift, i=True)     
+#
+#            dim = domain.item(axis)
+#            n = ((value - c.data[0]) / period).floor(i=True)
+#        #--- End: if
+#
+#        if dry_run:
+#            return  {'axis': axis, 'roll': shift, 'nperiod': n*period}
+#
+#        if n:
+#            np = n * period
+#            c += np
+#            if c.hasbounds:
+#                bounds = c.bounds
+#                bounds += np
+#        #--- End: if
+#                
+#        return c
+#    #--- End: def
+
     def asdimension(self, copy=True):
         '''
 
@@ -2812,14 +2895,14 @@ None
                 value = value.override_units(self.Units)
             elif not units.equivalent(self.Units):
                 raise ValueError(
-"Period units (%r) are not equivalent to coordinate units (%r)" %
-units, self.Units)
+"Period units {!r} are not equivalent to coordinate units {!r}".format(
+    units, self.Units))
 
             range = self.Data.range()
             if range >= value:
                 raise ValueError(
-"The coordinates' range (%r) is not less than the period (%r)" %
-range, value)
+"The coordinate range {!r} is not less than the period {!r}".format(
+    range, value))
         #--- End: if
 
         self._period = value
