@@ -44,12 +44,12 @@ from psutil      import virtual_memory, Process
 
 from .constants import CONSTANTS, _file_to_fh
 
-# Are we running on Linux?
+# Are we running on GNU/Linux?
 _linux = system() == 'Linux'
 
 if _linux:
     # ----------------------------------------------------------------
-    # LINUX
+    # GNU/LINUX
     # ----------------------------------------------------------------
     _meminfo_fields = set(('SReclaimable:', 'Cached:', 'Buffers:', 'MemFree:'))
     _meminfo_file   = open('/proc/meminfo', 'r', 1)
@@ -57,7 +57,7 @@ if _linux:
     def _free_memory():
         '''
 
-The amount of available physical memory on Linux.
+The amount of available physical memory on GNU/Linux.
 
 This amount includes any memory which is still allocated but is no
 longer required.
@@ -98,7 +98,7 @@ longer required.
     #--- End: def
 else:
     # ----------------------------------------------------------------
-    # NOT LINUX
+    # NOT GNU/LINUX
     # ----------------------------------------------------------------
     def _free_memory():
         '''
@@ -493,7 +493,7 @@ _max_number_of_open_files = resource.getrlimit(resource.RLIMIT_NOFILE)[0]
 
 if _linux:
     # ----------------------------------------------------------------
-    # LINUX
+    # GNU/LINUX
     # ----------------------------------------------------------------
 
     # Directory containing a symbolic link for each file opened by the
@@ -504,7 +504,7 @@ if _linux:
         '''
 
 Return True if the total number of open files is greater than the
-current threshold. LINUX.
+current threshold. GNU/LINUX.
 
 The threshold is defined as a fraction of the maximum possible number
 of concurrently open files (an operating system dependent amount). The
@@ -535,7 +535,7 @@ False
     #---End: def
 else:
     # ----------------------------------------------------------------
-    # NOT LINUX
+    # NOT GNU/LINUX
     # ---------------------------------------------------------------- 
     _process = Process(getpid())
 
