@@ -1651,8 +1651,8 @@ created. The ``seen`` dictionary is updated for *cfvar*.
                 **g['compression'])
         except (TypeError, RuntimeError):
             raise TypeError(
-                "Can't write field with data type %s from %r to a %s file" %
-                (datatype, cfvar, g['netcdf'].file_format))
+                "Can't write data type {0!r} from {1!r} to a {2} file".format(
+                    datatype, cfvar, g['netcdf'].file_format))
 
         _write_attributes(g['nc'][ncvar], netcdf_attrs)
 
