@@ -4,6 +4,7 @@ from numpy import empty as numpy_empty
 from numpy import where as numpy_where
 from numpy import sum as numpy_sum
 from .data.data import Data
+from .coordinate import DimensionCoordinate
 from .functions import REGRID_LOGGING
 from . import _found_ESMF
 if _found_ESMF:
@@ -109,7 +110,7 @@ after every call to ESMPy.
     #--- End: def
 
     @staticmethod
-    def create_grid(lon, lat, cyclic, mask=None):
+    def create_grid(lon, lat, cyclic, mask=None, use_bounds=True):
         """
 
 Create an ESMPy grid given 1D latitude and 1D longitude coordinates
@@ -134,6 +135,10 @@ grid may have an associated mask.
         to mask.  Where the elements of mask are True the output grid
         is masked.
 
+    use_bounds : bool, optional
+        Whether to populate the grid corners with information from
+        the bounds or not. True by default.
+
 :Returns:
 
     out: ESMF.Grid
@@ -141,27 +146,33 @@ grid may have an associated mask.
         grid in regridding.
 
         """
-        # Check the bounds for contiguity if they exist
-        if lon.hasbounds and not lon.contiguous(overlap=False):
-            raise ValueError('The longitude bounds must be contiguous.')
+        if use_bounds:
+            # Check the bounds for contiguity if they exist
+            if lon.hasbounds and not lon.contiguous(overlap=False):
+                raise ValueError('The longitude bounds must be contiguous.')
+            #--- End: if
+            if lat.hasbounds and not lon.contiguous(overlap=False):
+                raise ValueError('The latitude bounds must be contiguous.')
+            #--- End: if
+            
+            # Get the bounds, creating them if they do not exist
+            x_bounds = lon.get_bounds(create=True)
+            y_bounds = lat.get_bounds(create=True).clip(-90, 90, 'degrees').array
+            
+            # If not set as cyclic already, check for cyclicity
+            if not cyclic:
+                cyclic = abs(x_bounds.datum(-1) - x_bounds.datum(0)) == Data(360,
+                         'degrees')
+            x_bounds = x_bounds.array
         #--- End: if
-        if lat.hasbounds and not lon.contiguous(overlap=False):
-            raise ValueError('The latitude bounds must be contiguous.')
-        #--- End: if
-        
-        # Get the bounds, creating them if they do not exist
-        x_bounds = lon.get_bounds(create=True)
-        y_bounds = lat.get_bounds(create=True).clip(-90, 90, 'degrees').array
-        
-        # If not set as cyclic already, check for cyclicity
-        if not cyclic:
-            cyclic = abs(x_bounds.datum(-1) - x_bounds.datum(0)) == Data(360,
-                     'degrees')
-        x_bounds = x_bounds.array
         
         # Create empty grid
         max_index = numpy_array([lon.size, lat.size], dtype='int32')
-        staggerLocs = [ESMF.StaggerLoc.CORNER, ESMF.StaggerLoc.CENTER]
+        if use_bounds:
+            staggerLocs = [ESMF.StaggerLoc.CORNER, ESMF.StaggerLoc.CENTER]
+        else:
+            staggerLocs = [ESMF.StaggerLoc.CENTER]
+        #--- End: if
         if cyclic:
             grid = ESMF.Grid(max_index, num_peri_dims=1, staggerloc=staggerLocs)
         else:
@@ -175,22 +186,24 @@ grid may have an associated mask.
         gridYCentre = grid.get_coords(y, staggerloc=ESMF.StaggerLoc.CENTER)
         gridYCentre[...] = lat.array.reshape((1, lat.size))
         
-        # Populate grid corners
-        gridCorner = grid.coords[ESMF.StaggerLoc.CORNER]
-        if cyclic:
-            gridCorner[x][...] = x_bounds[:, 0].reshape(lon.size, 1)
-        else:
-            n = x_bounds.shape[0]
-            tmp_x = numpy_empty(n + 1)
-            tmp_x[:n] = x_bounds[:,0]
-            tmp_x[n] = x_bounds[-1,1]
-            gridCorner[x][...] = tmp_x.reshape(lon.size + 1, 1)
+        if use_bounds:
+            # Populate grid corners
+            gridCorner = grid.coords[ESMF.StaggerLoc.CORNER]
+            if cyclic:
+                gridCorner[x][...] = x_bounds[:, 0].reshape(lon.size, 1)
+            else:
+                n = x_bounds.shape[0]
+                tmp_x = numpy_empty(n + 1)
+                tmp_x[:n] = x_bounds[:,0]
+                tmp_x[n] = x_bounds[-1,1]
+                gridCorner[x][...] = tmp_x.reshape(lon.size + 1, 1)
+            #--- End: if
+            n = y_bounds.shape[0]
+            tmp_y = numpy_empty(n + 1)
+            tmp_y[:n] = y_bounds[:,0]
+            tmp_y[n] = y_bounds[-1,1]
+            gridCorner[y][...] = tmp_y.reshape(1, lat.size + 1)
         #--- End: if
-        n = y_bounds.shape[0]
-        tmp_y = numpy_empty(n + 1)
-        tmp_y[:n] = y_bounds[:,0]
-        tmp_y[n] = y_bounds[-1,1]
-        gridCorner[y][...] = tmp_y.reshape(1, lat.size + 1)
         
         # Add the mask if appropriate
         if not mask is None:
@@ -203,7 +216,8 @@ grid may have an associated mask.
     #--- End: def
     
     @staticmethod
-    def create_2Dgrid(lon, lat, x_order, y_order, cyclic, mask=None):
+    def create_2Dgrid(lon, lat, x_order, y_order, cyclic, mask=None,
+                      use_bounds=True):
         """
 
 Create an ESMPy grid given 2D latitude and 2D longitude coordinates
@@ -236,6 +250,10 @@ grid may have an associated mask.
         points to mask.  Where the elements of mask are True the
         output grid is masked.
 
+    use_bounds : bool, optional
+        Whether to populate the grid corners with information from
+        the bounds or not. True by default.
+
 :Returns:
 
     out: ESMF.Grid
@@ -253,7 +271,7 @@ grid may have an associated mask.
                              ' have the same shape.')
         
         # Check whether bounds exist or not, and get them if they do
-        hasbounds = lon.hasbounds and lat.hasbounds
+        hasbounds = lon.hasbounds and lat.hasbounds and use_bounds
         if hasbounds:
             x_bounds = lon.bounds
             y_bounds = lat.bounds.clip(-90, 90, 'degrees')
@@ -321,69 +339,115 @@ grid may have an associated mask.
         return grid, hasbounds
     #--- End: def
 
-#    @staticmethod
-#    def create_cartesian_grid(coords, mask=None):
-#        """
-#
-#Create a cartesian grid with between 1 and 3 dimensions given a tuple or list
-#of dimension coordinates and optionally a mask. The number of coordinates
-#passed will determine the dimensionality of the grid.
-#
-#:Parameters:
-#
-#    coords : tuple or list of cf.DimensionCoordinate objects
-#        The coordinates specifying the grid. There must be between 1 and 3
-#        elements.
-#
-#    mask : numpy.ndarray, optional
-#        An optional numpy array of booleans containing the grid points to mask.
-#        Where the elements of mask are True the output grid is masked.
-#
-#        """
-#        
-#        # Test the dimensionality of the list of coordinates
-#        ndim = len(coords)
-#        if ndim < 2 or ndim > 3:
-#            raise ValueError('Cartesian grid must have 2 or 3 dimensions.')
-#        #--- End: if
-#        
-#        shape = list()
-#        for coord in coords:
-#            shape.append(coord.size)
-#        
-#        # Initialize the grid
-#        max_index = numpy_array(shape, dtype='int32')
-#        staggerLocs = [ESMF.StaggerLoc.CORNER, ESMF.StaggerLoc.CENTER]
-#        grid = ESMF.Grid(max_index, coord_sys=ESMF.CoordSys.CART,
-#                         staggerloc=staggerLocs)
-#        
-#        # Populate the grid centres
-#        for d in xrange(0, ndim):
-#            gridDCentre = grid.get_coords(d, staggerloc=ESMF.StaggerLoc.CENTER)
-#            gridDCentre[...] = coords[d].array.reshape([shape[d] if x == d else 1
-#                                                       for x in xrange(0, ndim)])
-#        #--- End: for
-#        
-#        # Populate grid corners
-#        gridCorner = grid.coords[ESMF.StaggerLoc.CORNER]
-#        for d in xrange(0, ndim):
-#            if coords[d].hasbounds and not coords[d].contiguous(overlap=False):
-#                raise ValueError('Coordinate ' + d + ' does not have contiguous bounds.')
-#            #--- End: if
-#            boundsD = coords[d].get_bounds(create=True).array
-#            boundsD = numpy_append(boundsD[:, 0], boundsD[-1, 1])
-#            gridCorner[d][...] = boundsD.reshape([shape[d] + 1 if x == d else 1
-#                                                  for x in xrange(0, ndim)])
-#        
-#        # Add the mask if appropriate
-#        if not mask is None:
-#            gmask = grid.add_item(ESMF.GridItem.MASK)
-#            gmask[...] = 1
-#            gmask[mask] = 0
-#        #--- End: if
-#        
-#        return grid
-#    #--- End: def
+    @staticmethod
+    def create_cartesian_grid(coords, mask=None, use_bounds=True):
+        """
+
+Create a cartesian grid with between 1 and 3 dimensions given a
+tuple or list of dimension coordinates and optionally a mask. The
+number of coordinates passed will determine the dimensionality
+of the grid.
+
+:Parameters:
+
+    coords : tuple or list of cf.DimensionCoordinate objects
+        The coordinates specifying the grid. There must be between
+        1 and 3 elements.
+
+    mask : numpy.ndarray, optional
+        An optional numpy array of booleans containing the grid
+        points to mask. Where the elements of mask are True the
+        output grid is masked.
+
+    use_bounds : bool, optional
+        Whether to populate the grid corners with information from
+        the bounds or not. True by default.
+
+:Returns:
+
+    out: ESMF.Grid
+        The resulting ESMPy grid for use as a source or destination
+        grid in regridding.
+
+        """
+        
+        # Test the dimensionality of the list of coordinates
+        ndim = len(coords)
+        if ndim < 1 or ndim > 3:
+            raise ValueError('Cartesian grid must have between 1 and 3 ' +
+                             'dimensions.')
+        #--- End: if
+        if ndim == 1:
+            coords = [DimensionCoordinate(data=Data(0),
+                                          bounds=Data([-1e-6,1e-6]))] + coords
+            if mask is not None:
+                mask = mask[None,:]
+            ndim = 2
+        #--- End: if
+        
+        shape = list()
+        for coord in coords:
+            shape.append(coord.size)
+        
+        # Initialize the grid
+        max_index = numpy_array(shape, dtype='int32')
+        if use_bounds:
+            if ndim < 3:
+                staggerLocs = [ESMF.StaggerLoc.CORNER, ESMF.StaggerLoc.CENTER]
+            else:
+                staggerLocs = [ESMF.StaggerLoc.CENTER_VCENTER,
+                               ESMF.StaggerLoc.CORNER_VFACE]
+            #--- End: if
+        else:
+            if ndim < 3:
+                staggerLocs = [ESMF.StaggerLoc.CENTER]
+            else:
+                staggerLocs = [ESMF.StaggerLoc.CENTER_VCENTER]
+            #--- End: if
+        #--- End: if
+        grid = ESMF.Grid(max_index, coord_sys=ESMF.CoordSys.CART,
+                         staggerloc=staggerLocs)
+        
+        # Populate the grid centres
+        for d in xrange(0, ndim):
+            if ndim < 3:
+                gridCentre = grid.get_coords(d, staggerloc=ESMF.StaggerLoc.CENTER)
+            else:
+                gridCentre = grid.get_coords(d, staggerloc=ESMF.StaggerLoc.CENTER_VCENTER)
+            gridCentre[...] = coords[d].array.reshape([shape[d] if x == d else 1
+                                                       for x in xrange(0, ndim)])
+            #--- End: if
+        #--- End: for
+        
+        # Populate grid corners
+        if use_bounds:
+            if ndim < 3:
+                gridCorner = grid.coords[ESMF.StaggerLoc.CORNER]
+            else:
+                gridCorner = grid.coords[ESMF.StaggerLoc.CORNER_VFACE]
+            for d in xrange(0, ndim):
+                if coords[d].hasbounds and not coords[d].contiguous(overlap=False):
+                    raise ValueError('Coordinate ' + str(d) +
+                                     ' does not have contiguous bounds.')
+                #--- End: if
+                boundsD = coords[d].get_bounds(create=True).array
+                if shape[d] > 1:
+                    tmp = numpy_empty(shape[d] + 1)
+                    tmp[0:-1] = boundsD[:, 0]
+                    tmp[-1] = boundsD[-1, 1]
+                    boundsD = tmp
+                gridCorner[d][...] = boundsD.reshape([shape[d] + 1 if x == d else 1
+                                                      for x in xrange(0, ndim)])
+        
+        # Add the mask if appropriate
+        if not mask is None:
+            gmask = grid.add_item(ESMF.GridItem.MASK)
+            gmask[...] = 1
+            gmask[mask] = 0
+        #--- End: if
+        
+        return grid
+    #--- End: def
 
     @staticmethod
     def create_field(grid, name):
@@ -506,6 +570,7 @@ the original order.
     @staticmethod
     def get_latlong(f, name):
         """
+
 Retrieve the latitude and longitude coordinates of a field for
 regridding and the associated informarion required. If 1D lat/long
 coordinates are found then these are returned, otherwise 2D lat/long
@@ -604,6 +669,52 @@ coordinates are searched for and if found returned.
         return x, y, x_axis, y_axis, x_key, y_key, x_size, y_size, f_2D
     #--- End: def
     
+    @staticmethod
+    def get_cartesian_coords(f, name, axes):
+        '''
+
+Retrieve the specified cartesian dimension coordinates of a field and their
+corresponding keys.
+
+:Parameters:
+
+    f : Field
+        The field to retrieve dimension coordinates from.
+
+    name : string
+        A name to identify the field in error messages.
+
+    axes : iterable
+        Specifiers for the dimension coordinates to be retrieved. See
+        cf.Field.axes for details.
+
+:Returns:
+
+    axis_keys : list
+        A list of the keys of the dimension coordinates retrieved.
+
+    coords : list
+        A list of the dimension coordinates retrieved.
+
+        '''
+        axis_keys = []
+        for axis in axes:
+            tmp = f.axes(axis)
+            if len(tmp) != 1:
+                raise ValueError('Each item in axes keywrod must specify ' +
+                                 'exactly one axis.')
+            axis_keys.append(tmp.pop())
+        
+        coords = []
+        for key in axis_keys:
+            d = f.dim(key)
+            if d is None:
+                raise ValueError('No ' + name + ' dimension coordinate ' +
+                                 'matches key ' + key + '.')
+            coords.append(d)
+        
+        return axis_keys, coords
+
     @staticmethod
     def compute_mass_grid(valuefield, areafield, dofrac=False, fracfield=None, 
                       uninitval=422397696.):
